<?php

namespace BFE;

// Exit if accessed directly
defined('ABSPATH') || exit;

/**
 * Core
 */
class FrontUserSubmit
{

  /**
   * The init
   */
  public static function init()
  {

    define('FE_PLUGIN_URL', plugins_url('', __FILE__));
    define('FE_PLUGIN_DIR_PATH', plugin_dir_path(__FILE__));
    define('FE_Template_PATH', plugin_dir_path(__FILE__) . 'templates/');
    define('FE_TEXT_DOMAIN', 'front-editor');


    add_action('plugins_loaded', [__CLASS__, 'true_load_plugin_textdomain']);

    add_action('plugins_loaded', [__CLASS__, 'add_components']);

    /**
     * Если нету форм то не добовляет ajax исправить
     */
    add_filter('post_row_actions', [__CLASS__, 'add_link_to_edit_this_post'], 10, 2);

    add_action('BFE_activate', [__CLASS__, 'activate_user_ability_to_upload_files']);

    add_action('BFE_deactivate', [__CLASS__, 'disable_user_ability_to_upload_files']);
  }


  /**
   * Add Components
   */
  public static function add_components()
  {
    require_once FE_PLUGIN_DIR_PATH . '/functions.php';
    require_once FE_PLUGIN_DIR_PATH . '/inc/MenuSettings.php';
    require_once FE_PLUGIN_DIR_PATH . '/inc/Shortcodes.php';
    require_once FE_PLUGIN_DIR_PATH . '/inc/PostsList.php';
    require_once FE_PLUGIN_DIR_PATH . '/inc/SavePost.php';
    require_once FE_PLUGIN_DIR_PATH . '/inc/Blocks.php';
    require_once FE_PLUGIN_DIR_PATH . '/inc/Editor.php';
    require_once FE_PLUGIN_DIR_PATH . '/inc/EditorWidget.php';
    require_once FE_PLUGIN_DIR_PATH . '/inc/Form.php';
    

    add_action('wp_enqueue_scripts', [__CLASS__, 'add_scripts']);
  }


  /**
   * Add languages
   *
   * @return void
   */
  public static function true_load_plugin_textdomain()
  {
    load_plugin_textdomain(FE_TEXT_DOMAIN, false, dirname(plugin_basename(__FILE__)) . '/languages/');
  }

  /**
   * Add scripts
   */
  public static function add_scripts()
  {
      $asset = require FE_PLUGIN_DIR_PATH . 'build/front.asset.php';

      wp_register_script('bfee-editor.js', FE_PLUGIN_URL . '/build/front.js', array('jquery'), $asset['version'], true);

      wp_register_style(
        'bfe-block-style',
        FE_PLUGIN_URL . '/build/frontStyle.css',
        [],
        $asset['version']
      );
  }

  /**
   * added link wp admin post archive
   *
   * @param [type] $actions
   * @param [type] $post
   * @return void
   */
  public static function add_link_to_edit_this_post($actions, $post)
  {
    if ($post->post_type !== 'fe_post_form') {
      if ($edit_link = Editor::get_post_edit_link($post->ID)) {
        $actions['bfe_front_editor_link'] = sprintf(
          '<a target="_blank" style="color:#388ffe;" href="%s">%s</a>',
          $edit_link,
          __('Edit in front editor', FE_TEXT_DOMAIN)
        );
      }
    }

    /**
     * Changing postForm archive page edit link
     */
    if ($post->post_type === 'fe_post_form') {
      $actions['edit'] = sprintf(
        '<a target="_blank" href="%s">%s</a>',
        home_url(sprintf('/wp-admin/edit.php?page=fe-post-forms&action=edit&id=%s', $post->ID)),
        __('Edit', FE_TEXT_DOMAIN)
      );

      unset($actions['inline hide-if-no-js']);
    }

    return $actions;
  }

  public static function activate_user_ability_to_upload_files()
  {
    $contributor = get_role('subscriber');
    $contributor->add_cap('upload_files');
  }

  public static function disable_user_ability_to_upload_files()
  {
    $contributor = get_role('subscriber');
    $contributor->remove_cap('upload_files');
  }
}

FrontUserSubmit::init();
