export default ($, Swal) => {
    // Make plugin menu active if we in editing page
    $('li.wp-has-current-submenu').removeClass('wp-has-current-submenu')
    $('li.toplevel_page_front_editor_settings').addClass('wp-has-current-submenu')

    $('.nav-tab.top').click((ev) => {
        $('.nav-tab.top').removeClass('nav-tab-active')
        $('.group.top').removeClass('active')

        $(ev.target).addClass('nav-tab-active')
        $($(ev.target).attr('href')).addClass('active')
    });

    $('.nav-tab.sub').click((ev) => {
        $('.nav-tab.sub').removeClass('nav-tab-active')
        $('.group.sub').removeClass('active')

        $(ev.target).addClass('nav-tab-active')
        $($(ev.target).attr('href')).addClass('active')
    });


    let localizeData = window.fe_post_form_data,
        admin_form_builder_nonce = $('#admin_form_builder_nonce').val(),
        formBuilderContainer = false,
        current_forBuilder_controls,
        updated_forBuilder_controls,
        formBuilderOptions;

    let options = {
        notify: {
            error: function (message) {
                return Swal.fire(message, '', 'error');
            },
            success: function (message) {
                return Swal.fire(message, '', 'success');

            },
            warning: function (message) {
                return Swal.fire(message);
            }
        },
        onAddField: function (fieldId) {
            const currentFieldId = fieldId,
                field = $(`#${fieldId}`),
                field_type = field.attr('type'),
                field_count_in_form = $(`.form-field[type="${field_type}"]`).length;

            /**
             * Check for max of type exactable for form
             */
            if ((field_type in formBuilderOptions.temp_back)
                && ('max_in_form' in formBuilderOptions.temp_back[field_type])
                && (formBuilderOptions.temp_back[field_type].max_in_form < field_count_in_form)) {
                formBuilderContainer.actions.removeField(currentFieldId);
                Swal.fire(`Oops...`, formBuilderOptions.messages.max_fields_warning);
            }

            /**
             * Disable fields for pro version
             */
            if (formBuilderOptions.disableProFields.includes(field_type)) {

                formBuilderContainer.actions.removeField(currentFieldId);

                Swal.fire({
                    title: formBuilderOptions.messages.for_pro_title,
                    text: formBuilderOptions.messages.for_pro_message,
                    icon: 'info',
                    showCancelButton: true,
                    confirmButtonText: formBuilderOptions.messages.for_pro_button_text,
                }).then((result) => {
                    /* Read more about isConfirmed, isDenied below */
                    if (result.isConfirmed) {
                        window.open(formBuilderOptions.messages.for_pro_link, '_blank');
                    }
                })
            }

        }

    };



    /**
     * Activate form builder
     */
    $(window).on('load', () => {
        updateFormBuilder();
    })

    /**
     * Update form builder
     */
    function updateFormBuilder(action = 'load') {
        let post_type = $('#fe_settings_post_type').val(),
            formArray_data = {
                action: 'fe_get_formBuilder_data',
                post_id: localizeData.post_id,
                post_type: post_type,
                admin_form_builder_nonce: admin_form_builder_nonce
            },
            formData = new FormData();

        if (post_type) {
            Swal.fire('')
            Swal.showLoading()
        }


        for (var key in formArray_data) {
            formData.append(key, formArray_data[key]);
        }

        fetch(localizeData.rest_url_get, {
            method: 'POST',
            body: formData,
            headers: {
                'X-WP-Nonce': localizeData.nonce,
            }
        })
            .then(response => response.json())
            .then(data => {
                if (data.success) {
                    let response = data.data
                    let formBuilderId = `form-builder`;

                    $(`#${formBuilderId}`).remove();
                    $('.formBuilder-wrapper').empty().append(`<div id="${formBuilderId}"></div>`);

                    formBuilderOptions = response.formBuilder_options;

                    let templates_obj = formBuilderOptions.temp_back;

                    /**
                     * Creating templates using data form backend
                     */
                    Object.keys(templates_obj).map((key, index) => {
                        formBuilderOptions.templates[key] = (fieldData) => {
                            return {
                                field: templates_obj[key].field,
                                onRender: function () {
                                    $(document.getElementById(fieldData.name));
                                }
                            }
                        }
                    });
                    /**
                    * Init formBuilder
                    */
                    formBuilderContainer = $(`#${formBuilderId}`).formBuilder(
                        { ...options, ...formBuilderOptions }
                    );
                    formBuilderContainer.promise.then(formBuilder => {
                        // Remove controls on ajax request if there do not needed
                        builder_control_controls(formBuilderOptions);

                        // Add groups
                        add_groups()

                        // Adding data 
                        if (action === 'load') {
                            formBuilderContainer.actions.setData(response.formBuilderData);
                        }

                        // Disable pro attributes
                        formBuilderOptions.disable_attr.map((val) => {
                            $(document).find(val).prop('disabled', true)
                        });

                        // Change style if is not pro version fields
                        formBuilderOptions.disableProFields.map((val) => {
                            $(`.input-control[data-type="${val}"]`).css('opacity', '0.7');
                        });

                        Swal.close()
                    });
                } else {
                    Swal.fire('Oops', 'Something goes wrong try later', 'error')
                }
            }).catch()


    }

    /**
     * Saving form data
     */
    function save_form_data() {

        let formArray = $('#fe-fromBuilder').serializeArray(),
            formArray_data = objectifyForm(formArray),
            formData = new FormData();

        Swal.fire('')
        Swal.showLoading()

        formArray_data.formBuilderData = formBuilderContainer.actions.getData('json', true);
        formArray_data.action = 'save_post_front_settings';


        for (var key in formArray_data) {
            formData.append(key, formArray_data[key]);
        }

        fetch(localizeData.rest_update, {
            method: 'POST',
            body: formData,
            headers: {
                'X-WP-Nonce': localizeData.nonce,
            }
        })
            .then(response => response.json())
            .then(data => {
                if (data.success) {
                    let response = data.data
                    let message = response.message;
                    if (response.post_id) {
                        $('#post_id').val(response.post_id)
                        $('#fe-fromBuilder .settings-header.primary code').html(`[fe_form id="${response.post_id}"]`)
                    }
                    Swal.fire(message.title, message.message, message.status);

                } else {
                    Swal.fire('Oops', 'Something goes wrong try later', 'error')
                }
            }).catch()

    }

    function objectifyForm(formArray) {
        //serialize data function
        var returnArray = {};
        for (var i = 0; i < formArray.length; i++) {
            returnArray[formArray[i]['name']] = formArray[i]['value'];
        }
        return returnArray;
    }

    /**
     * Update for builder on post type update
     */
    $('#fe_settings_post_type').on('change', function (ev) {
        ev.preventDefault();
        updateFormBuilder('post_type_change');
    });

    /**
     * Save data
     */
    $('#save-form-post').on('click', function (ev) {
        ev.preventDefault();
        save_form_data();
    });

    /**
     * Remove controls on ajax request if there do not needed
     * @param {*} formBuilderOptions 
     */
    function builder_control_controls(formBuilderOptions) {
        current_forBuilder_controls = [];
        updated_forBuilder_controls = [];

        // Getting current controls
        $('.frmb-control li').each((index, elem) => {
            let data_type = $(elem).attr('data-type');
            current_forBuilder_controls.push(data_type)
        })

        /**
         * Creat array with updated controls
         */
        Object.keys(formBuilderOptions.temp_back).map((key, index) => {
            updated_forBuilder_controls.push(key)
        });

        console.log(updated_forBuilder_controls)

        /**
         *  default fields and custom fields
         */
        updated_forBuilder_controls = [...formBuilderOptions.defaultControls, ...updated_forBuilder_controls]
        console.log(updated_forBuilder_controls)
        /**
         * Find difference between controls
         */
        let difference = current_forBuilder_controls.filter(x => !updated_forBuilder_controls.includes(x));
        console.log(difference)
        difference.map((val) => {
            $(`[data-type="${val}"]`).remove();
        })
    }

    /**
     * Adding group fields
     */
    function add_groups() {
        let controls_group = formBuilderOptions.controls_group
        Object.keys(controls_group).map((key, index) => {
            $(`<p class="group-name ${key}">${controls_group[key].label}</p>`).insertBefore(`[data-type="${controls_group[key].types[0]}"]`);
        })
    }


}