'use strict';

import EditorJS from '@editorjs/editorjs';

import Header from '@editorjs/header';

import ImageTool from '@editorjs/image';

import Embed from '@editorjs/embed';

import Quote from '@editorjs/quote';

import Marker from '@editorjs/marker';

import CodeTool from '@editorjs/code';

//import LinkTool from '@editorjs/link';

import List from '@editorjs/list';

import Delimiter from '@editorjs/delimiter';

import InlineCode from '@editorjs/inline-code';

//import RawTool from '@editorjs/raw';

import Warning from '@editorjs/warning';

import Table from '@editorjs/table';

import Checklist from '@editorjs/checklist';

import WPImage from './class-wp-image';


export default ($) => {
    let editor_js,
        container_id,
        editor_data = window.editor_data,
        editor_js_settings;

    $(document).ready(function () {
        $('.EditorJS-editor').each((i, val) => {
            container_id = $(val).attr('id');
            editor_js_settings = get_editor_settings()
            editor_js = new EditorJS(editor_js_settings);
        })
    })

    try {
        editor_data = JSON.parse(editor_data);
    }
    catch (e) {
        editor_data = editor_data;
    }

    /**
     * Save on change data
     */
    function onChangeSaveData() {
        editor_js.save().then((data) => {
            document.getElementById(`${container_id}-textarea`).value = JSON.stringify(data);
        });
    }

    /**
     * Editor js params
     * @returns 
     */
    function get_editor_settings() {
        let editor_settings = editor_data.editor_settings;
        return {
            holder: container_id,
            placeholder: editor_data.translations.editor_field_placeholder,
            autofocus: true,
            i18n: editor_data.translations.i18n,
            tools: {
                ...(editor_settings.editor_header_plugin && {
                    header: {
                        class: Header,
                        inlineToolbar: true,
                        config: {
                            placeholder: 'Enter a header',
                            levels: [2, 3, 4],
                            defaultLevel: 3
                        },
                        shortcut: 'CMD+SHIFT+H'
                    }
                }),
                ...(editor_settings.editor_gallery_plugin && {
                    wpImageGallery: {
                        class: WPImage,
                        inlineToolbar: true,
                        config: {
                            wp_media_uploader: wpMediaUploader()
                        }
                    }
                }),
                ...(editor_settings.editor_image_plugin && {
                    image: {
                        class: ImageTool,
                        inlineToolbar: true,
                        config: {
                            uploader: {
                                uploadByFile(file) {
                                    return uploadImage(file).then(data => {
                                        if (!data.success) {
                                            editor_js.notifier.show({
                                                message: data.data.message ?? 'Something goes wrong try later',
                                                style: 'error',
                                            });
                                            return { "success": 0 };
                                        }
                                        return {
                                            "success": 1,
                                            "file": {
                                                "url": data.data.url,
                                            }
                                        };
                                    })
                                },
                                uploadByUrl(url) {
                                    return uploadImage(null, url).then(data => {
                                        if (!data.success) {
                                            editor_js.notifier.show({
                                                message: data.data.message ?? 'Something goes wrong try later',
                                                style: 'error',
                                            });
                                            return { "success": 0 };
                                        }
                                        return {
                                            "success": 1,
                                            "file": {
                                                "url": data.data.url,
                                            }
                                        };
                                    })
                                }

                            }

                        }
                    }
                }),

                ...(editor_settings.editor_list_plugin && {
                    list: {
                        class: List,
                        inlineToolbar: true,
                        shortcut: 'CMD+SHIFT+L'
                    }
                }),

                ...(editor_settings.editor_checklist_plugin && {
                    checklist: {
                        class: Checklist,
                        inlineToolbar: true,
                    }
                }),

                ...(editor_settings.editor_quote_plugin && {
                    quote: {
                        class: Quote,
                        inlineToolbar: true,
                        config: {
                            quotePlaceholder: 'Enter a quote',
                            captionPlaceholder: 'Quote\'s author',
                        },
                        shortcut: 'CMD+SHIFT+O'
                    },
                }),

                ...(editor_settings.editor_warning_plugin && {
                    warning: Warning,
                }),

                ...(editor_settings.editor_marker_plugin && {
                    marker: {
                        class: Marker,
                        shortcut: 'CMD+SHIFT+M'
                    }
                }),

                ...(editor_settings.editor_code_plugin && {
                    code: {
                        class: CodeTool,
                        shortcut: 'CMD+SHIFT+C'
                    },
                }),

                ...(editor_settings.editor_delimiter_plugin && {
                    delimiter: Delimiter
                }),

                ...(editor_settings.editor_inlineCode_plugin && {
                    inlineCode: {
                        class: InlineCode,
                        shortcut: 'CMD+SHIFT+C'
                    },
                }),

                ...(editor_settings.editor_embed_plugin && {
                    embed: Embed,
                }),

                ...(editor_settings.editor_table_plugin && {
                    table: {
                        class: Table,
                        inlineToolbar: true,
                        shortcut: 'CMD+ALT+T'
                    },
                }),

                //linkTool: LinkTool,
            },

            defaultBlock: 'paragraph',

            data: {
                blocks: editor_data.data.blocks
            },
            onReady: () => {
                if (!editor_data.data) {
                    editor_js.blocks.renderFromHTML(editor_data.html_post_content)
                        .catch(error => {
                            console.log('Error with rendering HTML data ' + error);
                        });
                }
            },
            onChange: () => {
                onChangeSaveData()
            }
        }

    };

    function objectifyForm(formArray) {
        //serialize data function
        var returnArray = {};
        for (var i = 0; i < formArray.length; i++) {
            returnArray[formArray[i]['name']] = formArray[i]['value'];
        }
        return returnArray;
    }

    /**
     * uploading image
     */
    function uploadImage(file = null, url = null) {
        return new Promise((resolve, reject) => {
            const formData = new FormData();
            let post_id = document.querySelector(`.fus-form`).getAttribute('post_id');
            console.log(post_id)

            formData.append('action', 'bfe_uploading_image')

            formData.append('post_id', post_id)

            if (file !== null) {
                formData.append('image', file)
            }
            if (url !== null) {
                formData.append('image_url', url)
            }

            fetch(editor_data.ajax_url, {
                method: 'POST',
                body: formData
            })
                .then(response => response.json())
                .then(data => {
                    console.log(data);
                    if (data.success) {
                        resolve(data);
                    } else {
                        resolve(data);
                    }
                }).catch()
        })
    }

    function wpMediaUploader() {
        var file_frame = 0;
        console.log(file_frame);
        if (file_frame) {
            file_frame.open();
            return;
        }

        // Create the media frame.
        file_frame = wp.media.frames.file_frame = wp.media({
            title: jQuery(this).data('uploader_title'),
            button: {
                text: jQuery(this).data('uploader_button_text'),
            },
            multiple: true,  // Set to true to allow multiple files to be selected
            type : 'image'//audio, video, application/pdf, ... etc
        });

        // When an image is selected, run a callback.
        file_frame.on('select', function () {
            // We set multiple to false so only get one image from the uploader
            var selection = file_frame.state().get('selection').first().toJSON();

            console.log(selection.url)

            // var attachment_ids = selection.map(function (attachment) {
            //     attachment = attachment.toJSON();
            //     var array = {id:attachment.id,url:attachment.url}
            //     return array;
            // }).join();

            //file_frame.open();
        });
        
    }
}
